/* Esercizio 1 lezione sui grafi:
   Trovare i punti di articolazione richiede
   trovare i nodi che "spezzano" la connettivita'
   del grafo.
   Inizialmente sappiamo che da 0 si puo' raggiungere
   qualsiasi nodo. Un nodo e' quindi di articolazione
   se la rimozione dei sui archi entranti e uscenti
   non permette di arrivre da 0 agli altri nodi */

#include <stdio.h>
#include <stdlib.h>


/* Legge una matrice di adiacenza. */
int **leggi(int n, int m)
{
    /* Allochiamo il vettore di righe. */
    int **matrice = malloc(sizeof(int*)*n);
    int i, u, v;

    /* Creiamo ognuna delle righe e azzeriamola */
    for ( i = 0; i < n; i++ )
    {
      matrice[i] = malloc(sizeof(int)*n);
      for ( u = 0; u < n; u++ )  
        matrice[i][u] = 0;
    }

    /* Scandiamo le coppie di interi che rappresentano
       l'arco (i, j). A questo punto settiamo matrice[i][j] = 1. */
    for ( i = 0; i < m; i++ )
    {
        scanf("%d%d", &u, &v);
        matrice[u][v] = 1;
    }

    return matrice;
}


/* Questa e' una visita che evita il nodo "candidato".
   E' una visita in profondita' (DFS) implementata in modo
   ricorsivo.
   I primi due parametri sono la matrice di adiacenza e il
   numero di nodi, poi viene il nodo corrente da visitare,
   poi il nodo da ignorare e poi l'array in cui
   si segnano i nodi gia' visitati. */
void visita_evita(int **grafo, int n, int nodo, int candidato, int *visitato) 
{
   int j;

   for ( j = 0; j < n; j++ ) 
     if ( j != candidato && grafo[nodo][j] && !visitato[j] )
     {
        visitato[j] = 1;
        visita_evita(grafo, n, j, candidato, visitato);
     }
}


/* Chiamata principale che  esegue la dfs: azzera
   l'array di oggetti visitati, segna la radice
   come controllata e fa partire la visita.
   Al termine, controlla che tutti gli n nodi siano stati
   raggiunti dalla visita. Se no, allora il candidato
   e' un punto di articolazione e ritorna 1, altrimenti 0. */
int dfs_evita(int **grafo, int n,  int candidato, int *visitato)
{

   int i;

   for ( i = 0; i < n; i++ ) 
     visitato[i] = 0;

   visitato[0] = 1;
   visita_evita(grafo, n, 0, candidato, visitato);

   for ( i= 0; i < n; i++ )
     if ( ! visitato[i] && i != candidato ) 
     {
        /*fprintf(stderr, "Testimone di %d: %d\n", candidato, i );*/
        return 1;
      }
   return 0;
}


main()
{
   int **grafo;
   int *pila, *visitato;
   int n, m, i;

   scanf("%d%d", &n, &m); 
   grafo = leggi(n, m);
  
   visitato = malloc(sizeof(int)*n);

   for ( i = 1; i < n; i++ ) 
    if ( dfs_evita( grafo, n,  i, visitato) )
      printf("%d ", i);   
   printf("\n");
   return 0;
}
