/** Esericizio 2 sui grafi: trovare il diametro
    di un grafo non orientato.
    Questo viene fatto calcolando il massimo tra
    la distanza tra tutte le coppie di nodi. Da lezione,
    si sa che per calcolare la distanza minima in un grafo
    non orientato da un certo nodo X a tutti gli altri
    nodi nel grafo (supponendo sia connesso), si puo' 
    usare una visita in ampiezza (BFS).
    Pertanto, si esegue una visita in ampiezza a partire
    da ogni nodo. Per ogni visita, manteniamo soltanto
    il valore massimo che troviamo. Poi, facciamo il massimo
    tra i risultati di tutte le visite. */
#include <stdio.h>
#include <stdlib.h>


/** Legge una matrice di adiacenza per un grafo
    non orientato. In questo modo, se (u,v) compare
    nell'input, sia matrice[u][v] che matrice[v][u]
    vengono messi a true. */

int **leggi_nonor(int n, int m)
{
    int **matrice = malloc(sizeof(int*)*n);
    int i, u, v;

    for ( i = 0; i < n; i++ )
    {
      matrice[i] = malloc(sizeof(int)*n);
      for ( u = 0; u < n; u++ )  
        matrice[i][u] = 0;
    }

    for ( i = 0; i < m; i++ )
    {
        scanf("%d%d", &u, &v);
        matrice[u][v] = matrice[v][u] = 1;
    }

    return matrice;
}


/* Aggiorna e' una visita in ampiezza a partire da una certa sorgente.
   Il primo parametro e' il grafo, il secondo il numero di elementi nel grafo,
   il terzo un array di distanze che viene riempito con le distanze
   dalla sorgente, e c e' un array che viene usato per gestire la coda
   (in modo da averne uno soltanto). */
int aggiorna(int **grafo, int n, int *distanze, int *visitato, int *c,  int sorgente)
{
   int testa = 0; /* Testa della coda. */
   int coda = 0; /* Coda della coda. */
   int i, j, nodo, max_dist = -1; 

   for ( i= 0; i < n; i++ ) { 
      visitato[i] = 0 ;
      distanze[i] = -1;
   }

   /* Iniziamo: mettiamo in testa la sorgente segnandola
      come visitata. La sorgente ha distanza 0 da se stessa. */
   c[testa++] = sorgente;
   visitato[sorgente] = 1;
   distanze[sorgente] = 0;

   /* Finche' la coda non e' vuota.. */
   while ( testa - coda ) 
   {
      /* .. esploriamo un nuovo nodo.  */
      nodo = c[coda++];
      for ( j = 0; j < n; j++ )
        if ( grafo[nodo][j] && ! visitato[j] )          
        {
          /* Inseriamo nuovi elementi nella coda.  
             Per le proprieta' di visite e BFS,
             sappiamo che questa e' la prima volta
             che scopriamo il vertice j, e che quindi
             l'arco (nodo, j) e' quello che ci porta
             la distanza minima. Pertanto, la distanza
             di j dalla sorgente e' dist[nodo] + 1. */
           visitato[j] = 1;
           c[testa++] = j;
           distanze[j] = distanze[nodo] + 1;
           if ( distanze[j] > max_dist) 
             max_dist = distanze[j];
        }
   }
   /* Massimo. */
   return max_dist;
}


main()
{
   int **grafo, *distanze, *coda;
   int *pila, *visitato;
   int n, m, i, j, max;

   scanf("%d%d", &n, &m); 
   grafo = leggi_nonor(n, m);

   distanze = malloc(sizeof(int)*n);
   coda = malloc(sizeof(int)*n);
   for( i = 0; i < n; i++ ) 
     distanze[i] = -1;
    
  
   visitato = malloc(sizeof(int)*n);
   max = -1;

   for ( i = 0; i < n; i++ ) 
   {
     j = aggiorna( grafo, n, distanze, visitato, coda, i ); 
     if ( j > max ) max = j;
   }

   printf("%d\n", max);
   return 0;
}
